/*
 *  Copyright (C) 2000, Sealevel Systems
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  The author may be contacted via email at support@sealevel.com.
 */
#include "seaio_driver.h"

#ifdef DEBUG
static char *SEAIO_CARDS_C __attribute__ ((unused)) =
    "$Id: seaio_cards.c,v 1.4 2005/01/24 16:26:35 dalew Exp $";
#endif

/*
 * A list of all our currently supported devices:
 * format is:
 * { card #, total ports, readable port count, writable port count, io base, 
 * io address space size, irq level, interrupt control port,
 * {input ports},  {output ports}, {control ports}, sample interval, 
 * {control words}, readsInverted?1:0, PCI?1:0}
 * 
 * Note1: If a port is an output, it also acts as an input -- returning 
 * whatever value was last written to it.  If a port is only an input, though, 
 * it cannot act as an output.
 * Note2: Note1 does not apply to the 8001, or any 8255-based cards (8005 and 
 * 4030, as of this writing).  In the case of cards with selectable input/
 * output ports, you cannot read a value from an output port.  
 *
 * readsInverted is to deal with some oddball cards which, when a value 
 * is written to an output port, will read back the inverse of that value.  If 
 * you suspect that is why your program isn't working, try
 * readsInverted.
 * 
 * Each array of ports (input, output, and control) is terminated with the 
 * value -1.
 * 
 */ 

static Seaio_Device seaio_cards[] = {
	{3093, 
		4, // total available ports
		2, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{0x3093, 
		4, // total available ports
		2, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{3094, 
		2, // total available ports
		2, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{-1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{0x3094, 
		2, // total available ports
		2, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{-1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{3095, 
		2, // total available ports
		0, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{0x3095, 
		2, // total available ports
		0, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{3096, 
		2, // total available ports
		1, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{0x3096, 
		2, // total available ports
		1, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{3098, 
		4, // total available ports
		0, // ports which can act as inputs
		4, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{0, 1, 2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{0x3098, 
		4, // total available ports
		0, // ports which can act as inputs
		4, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{0, 1, 2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{3710, 
		2, // total available ports
		0, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{0x3710, 
		2, // total available ports
		0, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{3720, 
		2, // total available ports
		2, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{-1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{0x3720, 
		2, // total available ports
		2, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{-1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{3730, 
		4, // total available ports
		2, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{0x3730, 
		4, // total available ports
		2, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
   {3810,
		7, // total available ports
		7, // ports which can act as inputs
		5, // ports which can act as outputs
		0, // io base (always 0 initially)
		16, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{0, 1, 2, 4, 5, 6, 7, -1}, // input port locations
		{0, 4, 5, 6, 7, -1}, // output port locations
		{4, 6, -1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
   {0x3810,
		7, // total available ports
		7, // ports which can act as inputs
		5, // ports which can act as outputs
		0, // io base (always 0 initially)
		16, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{0, 1, 2, 4, 5, 6, 7, -1}, // input port locations
		{0, 4, 5, 6, 7, -1}, // output port locations
		{4, 6, -1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
   {3820,
		13, // total available ports
		7, // ports which can act as inputs
		11, // ports which can act as outputs
		0, // io base (always 0 initially)
		16, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{0, 1, 2, 4, 5, 6, 7, -1}, // input port locations
		{0, 4, 5, 6, 7, 8, 9, 10 ,11, 12, 13, -1}, // output port locations
		{4, 6, -1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
   {0x3820,
		13, // total available ports
		7, // ports which can act as inputs
		11, // ports which can act as outputs
		0, // io base (always 0 initially)
		16, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{0, 1, 2, 4, 5, 6, 7, -1}, // input port locations
		{0, 4, 5, 6, 7, 8, 9, 10 ,11, 12, 13, -1}, // output port locations
		{4, 6, -1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		0  // is pci?
	},
	{4030, 
		6, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		8, // IO address space size
		0, // irq level (always 0 initially)
		{3, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 7, -1}, // control port locations
		0,   // placeholder for sample interval
		{2}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{0x4030, 
		6, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		8, // IO address space size
		0, // irq level (always 0 initially)
		{3, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 7, -1}, // control port locations
		0,   // placeholder for sample interval
		{2}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		0  // is pci?
	},
	{8001, 
		4, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		8, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{4, -1}, // control port locations
		0,   // placeholder for sample interval
		{1}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
	{0x8001, 
		4, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		8, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{4, -1}, // control port locations
		0,   // placeholder for sample interval
		{1}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
	{8002, 
		2, // total available ports
		1, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{0x8002, 
		2, // total available ports
		1, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{8003, 
		2, // total available ports
		0, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
	{0x8003, 
		2, // total available ports
		0, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
	{8004, 
		4, // total available ports
		2, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{0x8004, 
		4, // total available ports
		2, // ports which can act as inputs
		2, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{8005, 
		6, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		13, // IO address space size
		0, // irq level (always 0 initially)
		{4, 12, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 11, -1}, // control port locations
		0,   // placeholder for sample interval
		{2}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
	{0x8005, 
		6, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		13, // IO address space size
		0, // irq level (always 0 initially)
		{4, 12, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 11, -1}, // control port locations
		0,   // placeholder for sample interval
		{2}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
	{8006, 
		2, // total available ports
		2, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{-1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{0x8006, 
		2, // total available ports
		2, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		2, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{-1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{8007, 
		4, // total available ports
		0, // ports which can act as inputs
		4, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{0, 1, 2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{0x8007, 
		4, // total available ports
		0, // ports which can act as inputs
		4, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{-1}, // interrupt control port
		{-1}, // input port locations
		{0, 1, 2, 3, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{8008, 
		4, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, -1}, // control port locations
		0,   // placeholder for sample interval
		{1}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{0x8008, 
		4, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, -1}, // control port locations
		0,   // placeholder for sample interval
		{1}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{8009, 
		12, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		29, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 11, 19, 27, -1}, // control port locations
		0,   // placeholder for sample interval
		{4}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{0x8009, 
		12, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		29, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 11, 19, 27, -1}, // control port locations
		0,   // placeholder for sample interval
		{4}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{8010, 
		4, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		8, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{4, -1}, // control port locations
		0,   // placeholder for sample interval
		{1}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{0x8010, 
		4, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		8, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{4, -1}, // control port locations
		0,   // placeholder for sample interval
		{1}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	}, 
	{8011, 
		2, // total available ports
		1, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{0x8011, 
		2, // total available ports
		1, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{8012, 
		2, // total available ports
		1, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{0x8012, 
		3, // total available ports
		2, // ports which can act as inputs
		1, // ports which can act as outputs
		0, // io base (always 0 initially)
		4, // IO address space size
		0, // irq level (always 0 initially)
		{5, -1}, // interrupt control port
		{0, 1, -1}, // input port locations
		{2, -1}, // output port locations
		{-1}, // control port locations
		0,   // placeholder for sample interval
		{0}, // control words
		{0}, // ucAIRange
		1, // reads inverted?
		1  // is pci?
	},
	{8014, 
		6, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		13, // IO address space size
		0, // irq level (always 0 initially)
		{4, 12, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 11, -1}, // control port locations
		0,   // placeholder for sample interval
		{2}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
	{0x8014, 
		6, // total available ports
		0, // ports which can act as inputs
		0, // ports which can act as outputs
		0, // io base (always 0 initially)
		13, // IO address space size
		0, // irq level (always 0 initially)
		{4, 12, -1}, // interrupt control port
		{-1}, // input port locations
		{-1}, // output port locations
		{3, 11, -1}, // control port locations
		0,   // placeholder for sample interval
		{2}, // control words
		{0}, // ucAIRange
		0, // reads inverted?
		1  // is pci?
	},
};
