/*
 *  Copyright (C) 2000, Sealevel Systems
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  The author may be contacted via email at support@sealevel.com.
 */
/*
 * $Header: /var/cvs/seaio/driver/kernel-2.6/seaio_driver.c,v 1.3 2005/01/20 21:08:13 dalew Exp $
 */

#include <linux/module.h>
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/interrupt.h>
#include <linux/pci.h>
#include <linux/fs.h>
#include <asm/system.h>
#include <asm/irq.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <linux/reboot.h> /* a whole set of replacement functions */
#include "seaio_driver.h"
#include "seaio_cards.c"

#ifdef DEBUG
static char *SEAIO_C __attribute__ ((unused)) =
    "$Id: seaio_driver.c,v 1.3 2005/01/20 21:08:13 dalew Exp $";
#endif

static char *driver_name = "SeaI/O Digital IO driver";
static char *driver_version = "$Revision: 1.3 $";

#define MAX_ISA_DEVICES 8
#define MAX_PCI_DEVICES 8
#define MAX_TOTAL_DEVICES 8
#define SEAIO_SUPPORTED_DEVICES_COUNT (sizeof(seaio_cards)/sizeof(Seaio_Device))

static int io[MAX_ISA_DEVICES] = { 0, };
static int irq[MAX_ISA_DEVICES] = { 0, };
static int cardtype[MAX_TOTAL_DEVICES] = { 0, };
static int debug_level = 0;
static int major = 0;

MODULE_PARM(cardtype, "1-" __MODULE_STRING(MAX_TOTAL_DEVICES) "i");
MODULE_PARM(io, "1-" __MODULE_STRING(MAX_ISA_DEVICES) "i");
MODULE_PARM(irq, "1-" __MODULE_STRING(MAX_ISA_DEVICES) "i");
MODULE_PARM(debug_level, "i");
MODULE_PARM(major, "i");

// The global linked list of SeaI/O devices
pSeaio_Struct Seaio_Device_List = NULL;
int SeaIO_Device_Count = 0;

void *seaio_get_text_ptr(void);
void *seaio_get_text_ptr()
{
	return seaio_get_text_ptr;
}

int init_module(void)
{
	int assigned_major = 0;
#ifdef EXPERIMENTAL_INTERRUPT_SUPPORT_ENABLED
	int assigned_major_buffer = 0;
#endif
	printk("%s %s\n", driver_name, driver_version);

	seaio_enumerate_devices();
	if (!Seaio_Device_List) {
		PDEBUG("Error: No SeaIO devices specified or found\n");

	} else {
		// Get the major number for the actual device node
		assigned_major = register_chrdev(major, "SeaIO", &seaio_fops);
		if (assigned_major < 0) {
			PDEBUG(KERN_WARNING "SeaIO: can't get major %d\n",
			       major);
			return assigned_major;
			}

		if (major == 0)
			major = assigned_major;
		#ifdef EXPERIMENTAL_INTERRUPT_SUPPORT_ENABLED
		// get the major number for the device's FIFO
		assigned_major_buffer = register_chrdev(major - 1, "SeaIO_FIFO", 
				&seaio_buffer_fops);
		if (assigned_major_buffer < 0) {
			PDEBUG(KERN_WARNING "SeaIO_FIFO: can't get major %d\n",
				major - 1);
			return assigned_major_buffer;
		}
		#endif
		
	}
	return 0;
}

void seaio_enumerate_devices(void)
{
	pSeaio_Struct info = NULL;
	int i = 0, j = 0;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("entering seaio_enumerate_devices\n");

	/* Check for user specified ISA devices */

	for (i = 0; (i < MAX_ISA_DEVICES) && io[i] && cardtype[i]; i++) {
		if (debug_level >= DEBUG_LEVEL_INFO)
			PDEBUG
			    ("ISA device specified io=%04X,irq=%d,cardtype=%04X\n",
			     io[i], irq[i], cardtype[i]);

		for (j = 0; j < SEAIO_SUPPORTED_DEVICES_COUNT; j++) {
			if (cardtype[i] == seaio_cards[j].model) {
				info =
				    seaio_allocate_device(seaio_cards[j]);

				if (!info) {
					/* error allocating device instance data */
					if (debug_level >=
					    DEBUG_LEVEL_ERROR)
						    PDEBUG
						    ("can't allocate device instance data.\n");
					continue;
				}
			}
			if (!info)
				continue;

			/* Copy user configuration info to device instance data */
			info->device_info.io_base = (unsigned int) io[i];
			info->device_info.irq_level = (unsigned int) irq[i];
			info->device_info.irq_level = info->device_info.irq_level;
			info->device_info.bus_type = SEAIO_BUS_TYPE_ISA;
			seaio_add_device(info);
			info = NULL;
		}
	}

#ifdef CONFIG_PCI
	/* Auto detect PCI adapters */
	seaio_find_pci_devices();
#endif
}

#ifdef CONFIG_PCI
void seaio_find_pci_devices(void)
{
	unsigned int io_base;
	unsigned char irq_line = 0;
	int i = 0, j = 0;
	pSeaio_Struct info = NULL;
	struct pci_dev *pdev,*pdev2;

	pdev = pdev2 = (struct pci_dev*) NULL;

	for (i = 0; i < SEAIO_SUPPORTED_DEVICES_COUNT; i++) {
		for (j = 0; j < MAX_PCI_DEVICES; j++) {
			if (seaio_cards[i].bus_type ==
					SEAIO_BUS_TYPE_PCI) {
				if	(
						(pdev = pci_find_device(SEALEVEL_VENDOR_ID,
														seaio_cards[i].model,
														pdev2)
						) 
						!= NULL
					)
				{
					// Set the from* to the one we found
					// so that we don't find it again.
					pdev2 = pdev;
					irq_line = pdev->irq;
					if
						(
						 pci_bus_read_config_dword
						 	(pdev->bus, pdev->devfn,
						  	PCI_BASE_ADDRESS_2,
						  	&io_base)
						 ) 
					{
						PDEBUG
							("%s(%d):I/O addr not set.\n",
							 __FILE__,
							 __LINE__);
						continue;
					}

					info = seaio_allocate_device (seaio_cards[i]);
					if (!info) {
						/* error allocating device instance data */
						if (debug_level >= DEBUG_LEVEL_ERROR)
							PDEBUG
								("can't allocate device instance data.\n");
						continue;
					}

					/* Copy user configuration info to device instance data */

					info->device_info.io_base =
						io_base &
						PCI_BASE_ADDRESS_IO_MASK;
					info->device_info.irq_level = (unsigned int) irq_line;
					info->device_info.irq_level = 
						irq_canonicalize(info->device_info.irq_level);
					info->device_info.bus_type = SEAIO_BUS_TYPE_PCI;
					/* add new device to device list */
					seaio_add_device(info);

					if (debug_level >= DEBUG_LEVEL_INFO)
						PDEBUG
							("PCI device specified io=%04X,irq=%d,cardtype=%04X\n",
							 info->device_info.io_base,
							 info->device_info.irq_level,
							 info->device_info.model);
				} 
				else 
				{
					// No more cards of this type
					break;
				}
			}
		}
	}
}
#endif				// CONFIG_PCI

pSeaio_Struct seaio_allocate_device(Seaio_Device type)
{
	int i = 0;
	pSeaio_Struct info =
	    (pSeaio_Struct) kmalloc(sizeof(Seaio_Struct), GFP_KERNEL);

	if (!info) {
		PDEBUG
		    ("seaio_allocate_device ERROR: can't allocate device instance data\n");
	} else {
		info->device_info.model = type.model;
		info->device_info.totalports = type.totalports;
		info->device_info.readable = type.readable;
		info->device_info.writable = type.writable;
		info->device_info.io_addr_size = type.io_addr_size;
		info->device_info.bus_type = type.bus_type;
		info->device_info.readsInverted = type.readsInverted;
		for (i = 0; i < MAXICP; i++) {
			info->device_info.interrupt_control_port[i] = 
				type.interrupt_control_port[i];
		}
		for (i = 0; i < MAXINPUTS; i++) {
			info->device_info.input[i] = type.input[i];
		}
		for (i = 0; i < MAXOUTPUTS; i++) {
			info->device_info.output[i] = type.output[i];
		}
		for (i = 0; i < MAXCONTROL; i++) {
			info->device_info.control[i] = type.control[i];
			info->device_info.controlwords[i] =
			    type.controlwords[i];
		}
		info->device_info.io_base = 0;
		info->io_addr_requested = 0;
		info->device_info.irq_level = 0;
		memset(info->device_name,0,100);
		strcpy(info->device_name,"SeaIO");
		info->irq_requested = 0;
		info->use_count = 0;
		info->ringBuffer = NULL;
		info->readPos = NULL;
		info->writePos = NULL;
		init_waitqueue_head(&(info->readWaitQueue));
		info->next = NULL;
	}
	return info;
}

void seaio_add_device(pSeaio_Struct info)
{
	if (debug_level > DEBUG_LEVEL_INFO) {
		PDEBUG("In seaio_add_device\n");
	}
	if (Seaio_Device_List == NULL) {
		Seaio_Device_List = info;
	} else {
		pSeaio_Struct current_dev = Seaio_Device_List;
		while (current_dev->next != NULL) {
			current_dev = current_dev->next;
		}
		current_dev->next = info;
	}
	SeaIO_Device_Count++;
}

int seaio_open(struct inode *inode, struct file *filp)
{
	int i = 0;
	int device_number = MINOR(inode->i_rdev);
	pSeaio_Struct info = NULL;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("seaio_open called\n");

	if (device_number > SeaIO_Device_Count - 1)
		return -ENODEV;

	info = Seaio_Device_List;
	for (i = 0; i < device_number; i++) {
		if (info->next != NULL)
			info = info->next;
		else
			PDEBUG
			    ("Walk off the end of the list in seaio_open!\n");
	}

	filp->private_data = info;

	if (info->use_count == 0) {
		/* we need to allocate resources, since nobody else is using us yet */
		if (seaio_allocate_resources(info) < 0) {
			//spit out an error message
			PDEBUG("SeaIO: Error allocating resources\n");
			return -ENODEV;
		}
	}
	info->use_count++;
	// Not used in 2.6 
	//MOD_INC_USE_COUNT;
	return 0;
}

int seaio_allocate_resources(pSeaio_Struct info)
{
	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("entering seaio_allocate_resources\n");

	if (info->use_count != 0) {
		/* card is already in use?  how did this happen? */
		PDEBUG
		    ("seaio warning: started to allocate resources, but the card appears to already be in use\n");
		return ERR_GEN;
	}

	if 
		(
			!request_region(info->device_info.io_base, 
								info->device_info.io_addr_size,
		       				"SeaIO")
		)
	{
		PDEBUG("%s(%d):I/O address conflict Addr=%08X, err=%d\n",
	       __FILE__, __LINE__, info->device_info.io_base, err);
		return -EIO;
	}
	if (debug_level > DEBUG_LEVEL_INFO)
		PDEBUG
		    ("Successfully allocated a %d-byte IO region at %04X\n",
		     info->device_info.io_addr_size, info->device_info.io_base);
	info->io_addr_requested = 1;
#ifdef EXPERIMENTAL_INTERRUPT_SUPPORT_ENABLED
	/* claim interrupt level */
	if (info->device_info.irq_level) {
		info->ringBuffer = (unsigned char *) 
			kmalloc((SAMPLE_BUFFERS + 1)*sizeof(unsigned short), GFP_KERNEL);
		if (info->ringBuffer == NULL) {
			PDEBUG("ERROR: Could not allocate interrupt sample buffer\n");
			seaio_deallocate_resources(info);
			return -ENODEV;
		} else {
			info->readPos = info->writePos = info->ringBuffer;
		}
		if (request_irq
		    (info->device_info.irq_level,seaio_interrupt,SA_SHIRQ,
		     info->device_name, info) < 0) {
			PDEBUG("%s(%d):Cant request interrupt -- IRQ=%d\n",
			       __FILE__, __LINE__, info->device_info.irq_level);
			seaio_deallocate_resources(info);
			return -ENODEV;
		}
		info->irq_requested = 1;
	}
	if (debug_level > DEBUG_LEVEL_INFO)
		PDEBUG
		    ("Successfully allocated an interrupt %d\n",
		     info->device_info.irq_level);
	info->irq_requested = 1;
#endif
	return OK;
}

int seaio_deallocate_resources(pSeaio_Struct info)
{
	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("entering seaio_deallocate_resources\n");
	if (info->use_count) {
		/* card is still in use?  how did this happen? */
		PDEBUG
		    ("seaio warning: tried to deallocate resources, but the card appears to still be in use\n");
		return ERR_GEN;
	}
	if (info->io_addr_requested) {
		release_region(info->device_info.io_base,
			       info->device_info.io_addr_size);
		info->io_addr_requested = 0;
		if (debug_level > DEBUG_LEVEL_INFO)
			PDEBUG("Deallocated a %d-byte IO region at %04X\n",
			       info->device_info.io_addr_size,
			       info->device_info.io_base);
	}
#ifdef EXPERIMENTAL_INTERRUPT_SUPPORT_ENABLED
	if (info->irq_requested) {
		// disable all interrupts:
		int i = 0;
		while (info->device_info.interrupt_control_port[i] > -1) {
			outb(info->device_info.io_base + 
				info->device_info.interrupt_control_port[i],0x00);
			i++;
		}
		free_irq(info->device_info.irq_level, info);
		info->irq_requested = 0;
		if (debug_level > DEBUG_LEVEL_INFO)
			PDEBUG("Deallocated an interrupt %d\n",
			       info->device_info.irq_level);
	}
	if (info->ringBuffer != NULL) {
		kfree(info->ringBuffer);
		info->ringBuffer = NULL;
		info->readPos = info->writePos = NULL;
	}
#endif
	return OK;
}

ssize_t
seaio_read(struct file * filp, char *buff, size_t size, loff_t * loff)
{
	pSeaio_Struct info = NULL;
	unsigned char CardState[32] = { 0 };
	int AdressableBytes = 0, i = 0;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("seaio_read called\n");

	info = (pSeaio_Struct) filp->private_data;
	AdressableBytes = info->device_info.io_addr_size;
	for (i = 0; i < AdressableBytes; i++) {
		// some cards return the complement of what is actually
		// on the port, so we invert this value before returning 
		if (info->device_info.readsInverted == 1)
			CardState[i] = ~inb(info->device_info.io_base + i);
		else
			CardState[i] = inb(info->device_info.io_base + i);
	}
	copy_to_user(buff, CardState, MIN(AdressableBytes, size));
	return MIN(AdressableBytes, size);
}

ssize_t
seaio_write(struct file * filp, const char *buff,
	    size_t size, loff_t * loff)
{
	pSeaio_Struct info = NULL;
	unsigned char CardState[32] = { 0 };
	int AdressableBytes = 0, i = 0;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("seaio_write called\n");

	info = (pSeaio_Struct) filp->private_data;
	AdressableBytes = info->device_info.io_addr_size;
	copy_from_user(CardState, buff, MIN(AdressableBytes, size));
	for (i = 0; i < MIN(AdressableBytes, size); i++) {
		outb(CardState[i], info->device_info.io_base + i);
	}
	return MIN(AdressableBytes, size);
}

int seaio_release(struct inode * inode, struct file * filp)
{
	pSeaio_Struct info = filp->private_data;

	filp->private_data = NULL;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("seaio_release called\n");
	// Not used in 2.6
	//MOD_DEC_USE_COUNT;
	info->use_count--;
	if (!info->use_count) {
		/* we need to deallocate the resources we were using */
		if (seaio_deallocate_resources(info) < 0) {
			//spit out an error code
			PDEBUG("SeaIO: Error deallocating resources\n");
		}
	}
#ifdef LINUX_20
	return;
#else
	return 0;
#endif
}

void seaio_handle_IOCWRITEPORT(struct file *filp, unsigned long arg)
{
	pSeaio_WritePortStruct	writePortStruct	= (pSeaio_WritePortStruct) arg;
	pSeaio_Struct			info			= NULL;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("seaio_handle_IOCWRITEPORT called\n");

	info = (pSeaio_Struct) filp->private_data;

	outb(writePortStruct->dataByte, info->device_info.io_base + writePortStruct->portOffset);
}

void seaio_handle_IOCREADPORT(struct file *filp, unsigned long arg)
{
	pSeaio_ReadPortStruct	readPortStruct	= (pSeaio_ReadPortStruct) arg;
	pSeaio_Struct			info			= NULL;
	unsigned char			dataByte;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("seaio_handle_IOCREADPORT called\n");

	info = (pSeaio_Struct) filp->private_data;

	dataByte = inb(info->device_info.io_base + readPortStruct->portOffset);

	copy_to_user(readPortStruct->dataByte, &dataByte, sizeof(unsigned char));
}

int
seaio_ioctl(struct inode *inode, struct file *filp,
	    unsigned int cmd, unsigned long arg)
{
	int err = 0, size = _IOC_SIZE(cmd);
	pSeaio_Struct info = filp->private_data;

	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("seaio_ioctl called cmd=%d, arg=%lu\n", cmd, arg);

	/* extract the type and number bitfields, and don't decode 
	 * wrong cmds: return EINVAL before verify_area()
	 */
	if (_IOC_TYPE(cmd) != SEAIO_IOC_MAGIC)
		return -EINVAL;
	if (_IOC_NR(cmd) > SEAIO_IOC_MAXNR)
		return -EINVAL;

	/*
	 * the direction is a bitmask, and VERIFY_WRITE catches R/W
	 * transfers.  'Type' is user-oriented, while 
	 * verify_area is kernel-oriented, so the concept of "read" and 
	 * "write" is reversed
	 */
	if (_IOC_DIR(cmd) & _IOC_READ)
		err = verify_area(VERIFY_WRITE, (void *) arg, size);
	else if (_IOC_DIR(cmd) & _IOC_WRITE)
		err = verify_area(VERIFY_READ, (void *) arg, size);
	if (err)
		return err;

	switch (cmd) {
	case SEAIO_IOCGDESCRIPTOR:
		copy_to_user((pSeaio_Device) arg,
			     (pSeaio_Device) & (info->device_info), size);
		break;

	case SEAIO_IOCSDESCRIPTOR:
		copy_from_user((pSeaio_Device) & (info->device_info),
			       (pSeaio_Device) arg, size);
		break;

	case SEAIO_IOCWRITEPORT:
		seaio_handle_IOCWRITEPORT(filp, arg);
		break;

	case SEAIO_IOCREADPORT:
		seaio_handle_IOCREADPORT(filp, arg);
		break;

	default:
		return -EINVAL;
	}
	return 0;
}

void cleanup_module(void)
{
	if (debug_level >= DEBUG_LEVEL_INFO)
		PDEBUG("cleanup_module called\n");
	seaio_deallocate_devices(Seaio_Device_List);
	unregister_chrdev(major, "SeaIO");
	unregister_chrdev(major - 1, "SeaIO_FIFO");
}

void seaio_deallocate_devices(pSeaio_Struct info)
{
	if (!info) {
		return;
	} else {
		if (info->next) {
			seaio_deallocate_devices(info->next);
		}
		if (debug_level > DEBUG_LEVEL_INFO)
			PDEBUG
			    ("Deallocating a device ... letting go of a %04X\n",
			     info->device_info.model);
		kfree(info);
		info = NULL;
	}
}

#ifdef EXPERIMENTAL_INTERRUPT_SUPPORT_ENABLED
#define INCREMENT_WRITE_POSITION 					\
info->writePos++; 							\
if (info->writePos > info->ringBuffer + SAMPLE_BUFFERS) { 		\
	info->writePos = info->ringBuffer; 				\
} 									\
if (info->writePos == info->readPos) { 					\
	PDEBUG("BUFFER OVERRUN!  Try increasing SAMPLE_BUFFERS in "	\
		"seaio_driver.h\n"); 					\
	info->readPos++; 						\
	if (info->readPos > info->ringBuffer + SAMPLE_BUFFERS) { 	\
		info->readPos = info->ringBuffer; 			\
	} 								\
}

static void seaio_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	pSeaio_Struct info = NULL;
	unsigned long flags = 0;

	unsigned char data0 = 0; // what was the state of our lines to sample?
	unsigned char data1 = 0; // what was the state of our lines to sample?
	unsigned char data2 = 0; // what was the state of our lines to sample?

	unsigned char cause = 0; // which port caused the interrupt?
	
	info = (pSeaio_Struct) dev_id;
	if (!info) {
		return;
	}
	
	save_flags(flags);
	cli();

	data0 = inb(info->device_info.io_base + 0);
	data1 = inb(info->device_info.io_base + 1);
	data2 = inb(info->device_info.io_base + 2);
	
	cause = inb(info->device_info.io_base + 5);
	
	if (cause != 0) {
		if (debug_level > DEBUG_LEVEL_INFO)
		PDEBUG("Got a real interrupt\n");
	} else {
		if (debug_level > DEBUG_LEVEL_INFO)
		PDEBUG("Got a fake interrupt\n");
		return;
	}
	
	*(info->writePos) = data0;
	INCREMENT_WRITE_POSITION;
	*(info->writePos) = data1;
	INCREMENT_WRITE_POSITION;
	*(info->writePos) = data2;
	INCREMENT_WRITE_POSITION;
	restore_flags(flags);
	
	// wake up seaio_read_buffer, if necessary
	wake_up_interruptible(&(info->readWaitQueue));
}

ssize_t seaio_read_buffer(struct file *filp, char *buff, size_t size, loff_t *offset)
{
	pSeaio_Struct info = (pSeaio_Struct) filp->private_data;
	unsigned long flags = 0;

	while (info->readPos == info->writePos) { // nothing to read right now...
		if (filp->f_flags & O_NONBLOCK) {
			return -EAGAIN;
		}
		interruptible_sleep_on(&(info->readWaitQueue));
	}

	save_flags(flags);
	cli();
	if (info->writePos > info->readPos) { // easy ... we haven't wrapped around
		if ((info->writePos - info->readPos) < size) {
			size = (info->writePos - info->readPos);
		}
		copy_to_user(buff,info->readPos,size);
		info->readPos += size;
		if (info->readPos > info->ringBuffer + SAMPLE_BUFFERS) {
			info->readPos = info->ringBuffer;
		}
	} else {
		if ((info->ringBuffer + SAMPLE_BUFFERS - info->readPos) < size) {
			size = info->ringBuffer + SAMPLE_BUFFERS - info->readPos;
		}
		copy_to_user(buff,info->readPos,size);
		info->readPos = info->ringBuffer;
	}
	restore_flags(flags);
	return size;
}
#endif


#ifdef MODULE_LICENSE
MODULE_LICENSE("GPL");
#endif
