#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "seaio.h"

// A place to store our original termios settings for when we leave canonical 
// mode.
static struct termios stored_settings;

// This fn takes the terminal out of canonical (line-oriented) mode and places
// it in a character-oriented mode.
void set_keypress(void) {
	struct termios new_settings;
	tcgetattr(0,&stored_settings);
	new_settings = stored_settings;
	new_settings.c_lflag &= (~ICANON);
	new_settings.c_cc[VTIME] = 0;
	new_settings.c_cc[VMIN] = 1;
	tcsetattr(0,TCSANOW,&new_settings);
	return;
}

// This fn returns the terminal to canonical mode.
void reset_keypress(void) {
	tcsetattr(0,TCSANOW,&stored_settings);
	return;
}

int main()
{
	int readitems = 10;
	unsigned char buffer[1500] = {0};
	unsigned char keepGoing = 1;
	int i = 0;
	int dev = 0;
	ADAPTER_STATE info = {0};
	fd_set rfds;
	struct timeval tv;
	int retval;
	int fd = open("/dev/diod0",O_NONBLOCK);
	set_keypress();
	SeaIo_OpenDevice(0,&dev);
	SeaIo_GetAdapterState(dev,&info);
	// ensure the card is set to all inputs
	info.ucModeCW[0] = 0x9B;
	info.ucModeCW[1] = 0x9B;
	SeaIo_SetAdapterState(dev,&info);
	// begin interrupt sampling
	SeaIo_StartInterruptSampling(dev);
	while(keepGoing) {
		// all this garbage is necessary to simply tell if, on a given
		// iteration, a key has been pressed.  I miss kbhit().
		FD_ZERO(&rfds);
		FD_SET(0, &rfds);
		tv.tv_sec = 0;
		tv.tv_usec = 10;
		retval = select(1,&rfds,NULL,NULL,&tv);
		if (retval) {
			if (getchar() == 'q') keepGoing = 0;
		} // finally we can get some real work done
		else {
			readitems = read(fd,buffer,sizeof(buffer));
			if (readitems < 0) continue;
			for (i = 0; i < (int) (readitems / sizeof(unsigned char)); i += 3) {
				printf("%02x %02x %02x\n",buffer[i],buffer[i+1],buffer[i+2]);
			}
			printf("\n");
			memset(buffer,0,sizeof(buffer));
		}
	}
	reset_keypress();
	close(fd);
	// It's important that we stop interrupt sampling here: some machines 
	// have been observed to lock up when a disconnected interrupt is 
	// triggered.  Yes, I know that's wrong.  No, I don't know any nicer
	// workaround.
	SeaIo_StopInterruptSampling(dev);
	SeaIo_CloseDevice(dev);
	return 0;
}
