/*
 *  Copyright (C) 2000, Sealevel Systems
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  The author may be contacted via email at dalew@sealevel.com.  This is 
 *  NOT a technical support address; contact your vendor for help installing
 *  and configuring the software.
 */

// SIODevice.cpp: implementation of the SIODevice class.
//
//////////////////////////////////////////////////////////////////////
#ifndef LINUX
#include <afxwin.h>
#include <winbase.h>
#include <winioctl.h>
#include "SeaIo32.h"
#else
#include "seaio.h"
#include "wincompat.h"
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#endif // ifdef LINUX
#include "seaiostruct.h"
#include "SIODevice.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

SIODevice::SIODevice()
{
#ifdef LINUX
	hDev = 0;
#else
	hDev = NULL;
#endif
}

SIODevice::~SIODevice()
{

}

SEAIO_ERROR_CODES SIODevice::Open(UCHAR DeviceNumber)
{
	ULONG status = SEAIO_ERROR_NONE;
	int i;

	if (hDev)
		SeaIo_CloseDevice(hDev);

	status = SeaIo_OpenDevice(DeviceNumber, &hDev);

	if (status == SEAIO_ERROR_NONE)
		status = SeaIo_GetAdapterInfo(hDev, &AdapterInfo);

	if (status == SEAIO_ERROR_NONE)
		status = SeaIo_GetAdapterState(hDev, &AdapterState);

	/* Setup A/D input ranges for all channels */
	SeaIo_GetAdapterState(hDev, &AdapterState);
	/* Initilize default A/D range for each channel */
	for(i=0; i<MAXAICHANNELS; i++)
		AdapterState.ucAIRange[i] = RANGE_0_TO_5;
	
	SeaIo_SetAdapterState(hDev, &AdapterState);

	return (SEAIO_ERROR_CODES)status;
}

UCHAR SIODevice::ReadAnalog(unsigned long Channel, unsigned long* ChData)
{
	int retCode;

	/* Read the specified A/D channel */
	retCode = SeaIo_ReadAnalog(hDev, Channel,	ChData);
	if( retCode != SEAIO_ERROR_NONE )
	{
		return retCode;
	}

	return SEAIO_ERROR_NONE;
}

UCHAR SIODevice::WriteAnalog(unsigned long Channel, unsigned long ChData)
{
	int retCode;

	/* Read the specified A/D channel */
	retCode = SeaIo_WriteAnalog(hDev, Channel,	ChData);
	if( retCode != SEAIO_ERROR_NONE )
	{
		return retCode;
	}

	return SEAIO_ERROR_NONE;
}


UCHAR SIODevice::ReadPort(UCHAR port)
{
	UCHAR buffer[4];
	DWORD length = 1;

	SeaIo_ReadReg(hDev, port, buffer, length, &length);

	return buffer[0];
}

SEAIO_ERROR_CODES SIODevice::WritePort(UCHAR port, UCHAR data)
{
	return (SEAIO_ERROR_CODES) SeaIo_WriteReg(hDev, port, data);

}

void SIODevice::Scan(int interval)
{
	int i = 0;
	if (AdapterInfo.wCardNumber == 8001)
	{
		for (i = 1; i < (AdapterInfo.ucPortOutCount * 8); i++)
		{
			SeaIo_RelayOpen(hDev, i);
			if (interval) Sleep(interval);
			SeaIo_RelayClose(hDev, i);
			if (interval) Sleep(interval);
		}

		for (i = (AdapterInfo.ucPortOutCount * 8); i > 0; i--)
		{
			SeaIo_RelayOpen(hDev, i);
			if (interval) Sleep(interval);
			SeaIo_RelayClose(hDev, i);
			if (interval) Sleep(interval);
		}
	}
	else
	{
		for (i = 1; i < (AdapterInfo.ucPortOutCount * 8); i++)
		{
			SeaIo_RelayClose(hDev, i);
			Sleep(interval);
			SeaIo_RelayOpen(hDev, i);
			Sleep(interval);
		}

		for (i = (AdapterInfo.ucPortOutCount * 8); i > 0; i--)
		{
			SeaIo_RelayClose(hDev, i);
			Sleep(interval);
			SeaIo_RelayOpen(hDev, i);
			Sleep(interval);
		}
	}
}

void SIODevice::Close()
{
#ifdef LINUX
	close(hDev);
#else
	SeaIo_CloseDevice(hDev);
#endif
}

void SIODevice::Count(int maxvalue, int delay)
{
	for (int i3 = 1; i3 <= 0x100; i3++) {
		for (int i2 = 1 ; i2 <= 0x100; i2++) {
			for (int i1 = 1; i1 <= 0x100; i1++) {
				for (int i0 = 0; i0 <= 0x100; i0++) {
					WriteRelative(0, i0);
					if (delay) Sleep(delay);
//					if ((i0+(i1*100)+(i2*0x10000)+(i3*0x1000000)) >= maxvalue)
//						return;
				}
				WriteRelative(0, 0);
				WriteRelative(1, i1);
			}
			WriteRelative(1, 0);
			WriteRelative(2, i2);
		}
		WriteRelative(2, 0);
		WriteRelative(3, i3);
	}
}

SEAIO_ERROR_CODES SIODevice::WriteRelative(UCHAR port, UCHAR data)
{
	UCHAR Buffer[2];
	DWORD status;

	Buffer[0] = port;
	if (AdapterInfo.wCardNumber == 8001)
		Buffer[1] = ~data;
	else
		Buffer[1] = data;

	status = SeaIo_PutData(hDev, Buffer);
	
	return (SEAIO_ERROR_CODES) status;
}

UCHAR SIODevice::SetupLoop(UCHAR mode=0)
{
	if (AdapterInfo.wCardNumber == 8001)
	{
		SeaIo_GetAdapterState(hDev, &AdapterState);
		AdapterState.ucModeCW[0] = mode;
		SeaIo_SetAdapterState(hDev, &AdapterState);
#ifndef LINUX
		SeaIo_SaveAdapterState(hDev);
#endif
		SeaIo_GetAdapterState(hDev, &AdapterState);
		SeaIo_GetAdapterInfo(hDev, &AdapterInfo);
		return 0;
	} else
		if (AdapterInfo.ucPortInCount == AdapterInfo.ucPortOutCount)
			return 0;
		else
			return 1;
}

UCHAR SIODevice::ReadRelative(UCHAR port)
{
	BYTE Buffer[20];
	DWORD dwLength = 20;

	if (port > dwLength)
		return 0;
	
	SeaIo_GetData(hDev, Buffer, dwLength, &dwLength);
	return Buffer[port];
}

void SIODevice::RunLoop(int i)
{
	int temp = 0;
	if (AdapterInfo.wCardNumber == 8001)
		SetupLoop(0x05);
	UCHAR PortToWrite = (UCHAR) (i % AdapterInfo.ucPortOutCount);
	UCHAR DataToWrite = (UCHAR) PortToWrite?i:(i>>1);
	WriteRelative( PortToWrite, DataToWrite );
	Sleep(1);
	if ((temp = ReadRelative( PortToWrite )) != (UCHAR) (DataToWrite))
		fprintf(stderr,"ERROR!!!! Loopback failed on port %d\n Expected to read %d, got %d instead\n", PortToWrite, DataToWrite, temp);

	// Little extra testing for the 8001 ...
	if (AdapterInfo.wCardNumber == 8001)
	{
		SetupLoop(0x0a);
		WriteRelative( PortToWrite, DataToWrite );
		Sleep(1);
		if ((temp = ReadRelative( PortToWrite )) != DataToWrite)
			fprintf(stderr,"ERROR!!!! Loopback failed on port %d\n Expected to read %d, got %d instead\n", PortToWrite, DataToWrite, temp);
	}
}

PUCHAR SIODevice::NotifyChange(PUCHAR pBuffer, DWORD *dwLength)
{
#ifndef LINUX
	DWORD dwRetStat;

	ResetEvent(ovrlpNotify.hEvent);
	SeaIo_NotifyInputChange(hDev, pBuffer, *dwLength, dwLength, &ovrlpNotify);
	dwRetStat = WaitForSingleObject(ovrlpNotify.hEvent, 10000);
	GetOverlappedResult(hDev, // handle of file, pipe, or communications device  
						&ovrlpNotify,	// address of overlapped structure 
						dwLength,    // address of actual bytes count 
						FALSE           // wait flag - false says do not wait 
						);
#endif // ifndef LINUX
	return pBuffer;
}

void SIODevice::SetAtoDRange(unsigned long Channel, int Range)
{
	if( Range >= 0x00 && Range < 0x04 )
	{	
		AdapterState.ucAIRange[Channel] = Range;	
		SeaIo_SetAdapterState(hDev, &AdapterState);
		SeaIo_GetAdapterInfo(hDev, &AdapterInfo);
		fprintf(stderr,"New channel %ld range is: %x \n",Channel,AdapterState.ucAIRange[Channel]);
	}
	else
	{
		fprintf(stderr,"INVALID RANGE! \n");
	}
}

void SIODevice::SetAdapterState(DWORD SampleInterval, UCHAR ModeCW[4])
{
	AdapterState.dwSampleInterval = SampleInterval;
	AdapterState.ucModeCW[0] = ModeCW[0];
	AdapterState.ucModeCW[1] = ModeCW[1];
	AdapterState.ucModeCW[2] = ModeCW[2];
	AdapterState.ucModeCW[3] = ModeCW[3];
	SeaIo_SetAdapterState(hDev, &AdapterState);
#ifndef LINUX
	SeaIo_SaveAdapterState(hDev);
#endif
	SeaIo_GetAdapterInfo(hDev, &AdapterInfo);
}

void SIODevice::SaveState()
{
#ifndef LINUX
	SeaIo_SaveAdapterState(hDev);
#endif
}

