/*
 *  Copyright (C) 2000, Sealevel Systems
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  The author may be contacted via email at dalew@sealevel.com.  This is 
 *  NOT a technical support address; contact your vendor for help installing
 *  and configuring the software.
 */
#ifndef LINUX
#include <afxwin.h>
#include "TestIt.h"
#include "iostream.h"
#include "seaio.h"
#include "SeaIo32.h"
#include "SIODevice.h"
#else
#include "wincompat.h"
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "SIODevice.h"
#include "TestIt.h"
#include "seaio.h"
#endif
SIODevice MyDevice;

int main(int argc, char * argv[])
{
	char buff[500];
	FILE * InputFile = stdin;
	SEAIO_ERROR_CODES retCode;

	if (argc > 1) // they must have wanted to give an input file
	{
		if ((InputFile = fopen(argv[1], "r")) == NULL) // the file was somehow invalid
		{
			fprintf(stderr,"Error:  Could not open file %s.  Taking console input instead.\n",argv[1]);
			InputFile = stdin;
		}
	} else // they didn't supply an input file
		InputFile = stdin;

	retCode = MyDevice.Open(0);
	if(retCode != SEAIO_ERROR_NONE)
	{
		fprintf(stderr,"Error:  Could not open device 0.\n");
		return 1;
	}

	if (InputFile == stdin)
		ShowMenu();
	
	while (fgets(buff, sizeof(buff), InputFile)) {
		if (InputFile == stdin)
			ShowMenu();
		Command *cmd = findCommand(*buff);
		if (cmd) {
			int arg1 = 0, arg2 = 0;
			if (cmd->ArgCount == 1 && !getOneArg(buff + 1, &arg1)) {
				fprintf(stderr, "Please enter a value for the following --\n");
				ShowLine(cmd->Arg1);
				fscanf(stdin,"%d",&arg1);
				fgetc(stdin);
			} else 
				if (cmd->ArgCount == 2 && !getTwoArgs(buff + 1, &arg1, &arg2)) {
					fprintf(stderr, "Please enter a value for the following --\n");
					ShowLine(cmd->Arg1);
					fscanf(stdin,"%d",&arg1);
					ShowLine(cmd->Arg2);
					fscanf(stdin,"%d",&arg2);
					fgetc(stdin);
				}

			if (cmd->ArgCount == 1) 
				cmd->func(*buff, arg1);
			else
				cmd->func(arg1, arg2);
		}
	}
	return 0;
}

char * Breakline(int LineLength, char * Destination, char * Source) {
	// do some stuff to break up the lines
	if (Source == NULL)
		return NULL;
	if ((DWORD)LineLength > strlen(Source)) {
		strcpy(Destination, Source);
		return NULL;
	}

	char Buffer[MAXLENGTH];
	char * pBuffer = Buffer;
	strcpy(pBuffer, Source);

	while (!isspace(*(pBuffer + LineLength))) LineLength--;
	pBuffer += LineLength;
	
//	for (DWORD i = LineLength; i < strlen(Source); i++)
		*pBuffer='\0';

	strcpy(Destination, Buffer);

	return (Source+(LineLength * sizeof(char)) + 1);
}

#define LINELENGTH 78
void ShowMenu(void) {
	int line = 0, linelength = LINELENGTH - 4;
	char buff[MAXLENGTH];
	char *placeholder = buff;
	char linebuff[LINELENGTH];
	char *shortline = linebuff;
#ifdef LINUX
	system("clear");
#else
	system("cls");
#endif

	fprintf(stderr,"\nMenu for SeaIOTst (Command line operation) \n");
	
	for (unsigned int i = 0; i < (sizeof(cmds) / sizeof(Command)); i++) { // run through commands
		if (strlen(cmds[i].Description) < (DWORD)linelength) // short line, this is easy
			fprintf(stderr,"%c - %s\n",cmds[i].cmd,cmds[i].Description);
		else { // long line
			line = 0;
			placeholder = buff;
			strcpy(placeholder, cmds[i].Description);
			do {
				placeholder = Breakline(linelength, shortline, placeholder);
				fprintf(stderr,"%c %c %s\n",(line == 0?cmds[i].cmd:' '),(line == 0?'-':' '),shortline);
				line++;
			} while (placeholder !=NULL);// end while (going through the long line)
		} // end else (long line)
	} // end for (run through commands)
}


Command *findCommand(int cmd) {
	unsigned int i;
	for (i = 0; i < (sizeof(cmds) / sizeof(Command)); i++) {
		if (cmds[i].cmd == toupper(cmd))
			return cmds + i;
	}

	fprintf(stderr, "Unknown Command '%c'\n", cmd);
	return 0;
}

int getOneArg(const char *buff, int *result) {
	return sscanf(buff, "%d", result) == 1;
}

int getTwoArgs(const char *buff, int *result1, int *result2) {
	return sscanf(buff, "%d %d", result1, result2) == 2;
}

void doStartSampling(int unused1, int unused2)
{
	SeaIo_StartInterruptSampling(MyDevice.hDev);
}

void doStopSampling(int unused1, int unused2)
{
	SeaIo_StopInterruptSampling(MyDevice.hDev);
}

void doReadPort(int unused, int port)
{
	UCHAR result;

	result = MyDevice.ReadPort(port);

	fprintf(stderr, "ReadReg returned %i (hex %x)\n", result, result);
}

void doWritePort(int port, int data)
{
	SEAIO_ERROR_CODES status=SEAIO_ERROR_NONE;
	if ((status = MyDevice.WritePort(port, data)) == SEAIO_ERROR_NONE)
		fprintf(stderr, "Wrote %d to port %d.\n", data, port);
	else
		printError(status);
}

void doScanPorts(int reps, int delay)
{
	fprintf(stderr,"Scanning %d times ... ",reps);
	for (int i = 0; i < reps; i++)
		MyDevice.Scan(delay);
	fprintf(stderr,"\n");
}

void doCountPorts(int maxvalue, int delay)
{
	fprintf(stderr,"Counting with a %dms delay (^C to stop)\n",delay);
	MyDevice.Count(maxvalue,delay);
}

void doWaitForPorts(int unused, int unused2)
{
	UCHAR Buffer[100] = {0};
	DWORD dwLength = 100;
	MyDevice.NotifyChange(Buffer, &dwLength);
	fprintf(stderr,"NotifyChange returned: ");
	for (DWORD i = 0; i < dwLength; i++)
		fprintf(stderr, "%x ",Buffer[i]);
	fprintf(stderr,"\n");
}

void doOpenPort(int unused, int port)
{
	SEAIO_ERROR_CODES status = SEAIO_ERROR_NONE;
	if ((status = MyDevice.Open((UCHAR)port)) == SEAIO_ERROR_NONE)
	{
	fprintf(stderr,"Opened adapter %d\n", port);
	fprintf(stderr,"Card Type: %d     Base I/O Address: 0x%x\n",MyDevice.AdapterInfo.wCardNumber,
			MyDevice.AdapterInfo.wBaseIO);
	fprintf(stderr,"Port Counts (Input/Output/Total):  %d / %d / %d\n",MyDevice.AdapterInfo.ucPortInCount,
			MyDevice.AdapterInfo.ucPortOutCount,MyDevice.AdapterInfo.ucPortCount);
	} else
	{
		fprintf(stderr,"Error opening device %d:\n",port);
		printError(status);
	}
}

void doQuit(int unused1, int unused2)
{
	MyDevice.Close();
	exit(0);
}

void doMonitorAtoD(int unused, int numchannels)
{
	DWORD ChData;
	int i;

	fprintf(stderr,"Requested %d channels \n",numchannels);
  /* Validate input parameter */
	if( numchannels > MAXAICHANNELS )
	{
		numchannels = MAXAICHANNELS;
	}

	for(i=0; i<numchannels; i++)
	{
		MyDevice.ReadAnalog(i, &ChData);
		fprintf(stderr,"Channel %d: Value: %lX \n",i,ChData);
	}
}

void doWriteDtoA(int Channel, int Data)
{
	fprintf(stderr,"Requested Channel %d Data %d \n",Channel, Data);
  /* Validate input parameter - only 2 channels of output on 3820 */
	if( Channel > 1 )
	{
		fprintf(stderr,"Invalid channel \n");
	}

	MyDevice.WriteAnalog(Channel, Data);
	fprintf(stderr,"Successfully updated channel %d \n",Channel);
}

void doSetAtoDRange(int unused, int range)
{
	int i;

	/* For now this routine sets all channels to the same A/D input range */
	for(i=0; i<MAXAICHANNELS; i++)
	{
		  fprintf(stderr,"Channel %d: Setting range to %X \n",i,range);        	
		  MyDevice.SetAtoDRange(i, range);
  }
}

void doLoopback(int repcount, int unused)
{
	if (MyDevice.SetupLoop(0))
	{
		fprintf(stderr,"Error: device not capable of loopback\n");
		return;
	}

	fprintf(stderr,"Performing %d loopback iterations ...",repcount);
	for (int i = 1; i < repcount*2; i++)
	{
		MyDevice.RunLoop(i);
	}
	fprintf(stderr,"\n");
}

void doSetState(int unused, int SampleInterval)
{
	UCHAR extraChar = 0;
	UCHAR MCW[4] = {0};
	switch(MyDevice.AdapterInfo.wCardNumber)
	{
		case 4030:
		case 8001:
		case 8005:
		case 8014:
		case 8008:
		case 8009:
		case 8010:
		case 8203:
		case 8205:
			fprintf(stderr,"Please enter the first Mode Control Word for this device (in decimal form):");
			fscanf(stdin,"%hhd",&MCW[0]);
			extraChar = 1;
	}
	switch(MyDevice.AdapterInfo.wCardNumber)
	{
		case 4030:
		case 8005:
		case 8014:
		case 8009:
		case 8203:
		case 8205:
			fprintf(stderr,"Please enter the second Mode Control Word for this device (in decimal form):");
			fscanf(stdin,"%hhd",&MCW[1]);
	}
	switch(MyDevice.AdapterInfo.wCardNumber)
	{
		case 8009:
		case 8205:
			fprintf(stderr,"Please enter the third Mode Control Word for this device (in decimal form):");
			fscanf(stdin,"%hhd",&MCW[2]);
	}
	switch(MyDevice.AdapterInfo.wCardNumber)
	{
		case 8009:
		case 8205:
			fprintf(stderr,"Please enter the fourth Mode Control Word for this device (in decimal form):");
			fscanf(stdin,"%hhd",&MCW[3]);
	}

	if (extraChar == 1) {
		fgetc(stdin);
	}

	MyDevice.SetAdapterState(SampleInterval, MCW);
	MyDevice.SaveState();
}

void doSaveState(int Unused1, int Unused2)
{
	MyDevice.SaveState();
}

void doGetAdapterInfo(int Unused1, int Unused2)
{
	fprintf(stderr,"Current Adapter Information:\n");
	fprintf(stderr,"Card Type: %d          Base I/O Address: 0x%x\n", MyDevice.AdapterInfo.wCardNumber, MyDevice.AdapterInfo.wBaseIO);
	fprintf(stderr,"Port Counts (Input/Output/Total):  %d / %d / %d\n", MyDevice.AdapterInfo.ucPortInCount, MyDevice.AdapterInfo.ucPortOutCount, MyDevice.AdapterInfo.ucPortCount);
	fprintf(stderr,"Sampling Interval: %ld \346s\n", MyDevice.AdapterState.dwSampleInterval);
}

void printError(DWORD dwError)
{
	switch (dwError)
	{
		case SEAIO_ERROR_PARAM:
			fprintf(stderr, "Returned error: SEAIO_ERROR_PARAM\n");
			break;		

		case SEAIO_ERROR_NOT_SUPPORTED:
			fprintf(stderr, "Returned error: SEAIO_ERROR_NOT_SUPPORTED\n"); 
			break;		
			
		case SEAIO_ERROR_PORT_NOT_OPEN:
			fprintf(stderr, "Returned error: SEAIO_ERROR_PORT_NOT_OPEN\n");
			break;		

		case SEAIO_ERROR_INVALID_NAME:
			fprintf(stderr, "Returned error: SEAIO_ERROR_INVALID_NAME\n");
			break;		
		
		case SEAIO_ERROR_SHARING_VIOLATION:
			fprintf(stderr, "Returned error: SEAIO_ERROR_SHARING_VIOLATION\n");
			break;		
			
		case SEAIO_ERROR_BUFFER:
			fprintf(stderr, "Returned error: SEAIO_ERROR_BUFFER\n");
			break;		
			
		case SEAIO_ERROR_INCOMPLETE:
			fprintf(stderr, "Returned error: SEAIO_ERROR_INCOMPLETE\n");
			break;		
			
		case SEAIO_ERROR_GENERAL:
			fprintf(stderr, "Returned error: SEAIO_ERROR_GENERAL\n");
			break;		
			
		case SEAIO_ERROR_IO_PENDING:
			fprintf(stderr, "Returned error: SEAIO_ERROR_IO_PENDING\n");
			break;		
						
		case SEAIO_ERROR_NO_TXBUFFER:
			fprintf(stderr, "Returned error: SEAIO_ERROR_NO_TXBUFFER\n");
			break;		
			
		case SEAIO_ERROR_INVALID_HANDLE:
			fprintf(stderr, "Returned error: SEAIO_ERROR_INVALID_HANDLE\n");
			break;		
			
		case SEAIO_ERROR_INVALID_DEVICE_REQUEST:
			fprintf(stderr, "Returned error: SEAIO_ERROR_INVALID_DEVICE_REQUEST\n");
			break;		
			
		case SEAIO_ERROR_LAST:
			fprintf(stderr, "Returned error: SEAIO_ERROR_LAST\n");
			break;		
		
		default:
			fprintf(stderr, "Returned unknown error: %ld\n", dwError);
	}
	return;
}

void ShowLine(char * instr)
{
	int line = 0, linelength = LINELENGTH;
	char buff[MAXLENGTH];
	char *placeholder = buff;
	char linebuff[LINELENGTH];
	char *shortline = linebuff;

	if (strlen(instr) < (DWORD)linelength) // short line, this is easy
		fprintf(stderr,"%s:",instr);
	else { // long line
		line = 0;
		placeholder = buff;
		strcpy(placeholder, instr);
		do {
			placeholder = Breakline(linelength, shortline, placeholder);
			fprintf(stderr,"%s%c", shortline, placeholder!=NULL?'\n':':');
			line++;
		} while (placeholder !=NULL);// end while (going through the long line)
	} // end else (long line)
}
