/*
 *  Copyright (C) 2000, Sealevel Systems
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  The author may be contacted via email at dalew@sealevel.com.  This is 
 *  NOT a technical support address; contact your vendor for help installing
 *  and configuring the software.
 */
#ifndef __TESTIT_H__
#define __TESTIT_H__

#define MAXLENGTH 300
typedef struct {
	char cmd;
	char Description[MAXLENGTH];
	char Arg1[MAXLENGTH];
	char Arg2[MAXLENGTH];
	int ArgCount;
	void (*func)(int, int);
} Command;

void ShowMenu(void);
void ShowLine(char *instr);
Command *findCommand(int cmd);
int getOneArg(const char * buff, int *result);
int getTwoArgs(const char * buff, int *result1, int *result2);
void doReadPort(int, int);
void doWritePort(int, int);
void doWaitForPorts(int, int);
void doScanPorts(int, int);
void doCountPorts(int, int);
void doOpenPort(int, int);
void doQuit(int, int);
void doLoopback(int, int);
void doMonitorAtoD(int, int);
void doWriteDtoA(int, int);
void doSetAtoDRange(int, int);
void doSetState(int, int);
void doSaveState(int, int);
void doGetAdapterInfo(int, int);
void doStartSampling(int, int);
void doStopSampling(int, int);
void printError(DWORD dwError);

static Command cmds[] = {
	{ 'C',	
		"Count in binary (hit Control-C to stop)", 
		"Maximum value to count to", 
		"Delay Interval (measured in milliseconds) (0 = do it as fast as possible)", 
		2, 
		doCountPorts },
	{ 'E',	
		"Set adapter state", 
		"Sample Interval (measured in microseconds)", 
		" ", 
		1, 
		doSetState },
	{ 'I',  
		"Get adapter information", 
		" ", 
		" ", 
		0, 
		doGetAdapterInfo},
	{ 'K',	
		"Walk Relays back and forth", 
		"Number times to walk up and down the row", 
		"Delay Interval (measured in milliseconds) (0 = do it as fast as possible)", 
		2,	
		doScanPorts	},
	{ 'L',	
		"Loopback Test", 
		"Number of loopback iterations to try", 
		"Please ensure the loopback plug is installed and type 'C'<enter> to continue", 
		2, 
		doLoopback },
	{ 'M',	
		"Monitor A/D Input",
		"Number of inputs to monitor",
		"",
		1,
		doMonitorAtoD },
	{ 'N',	
		"Write D/A Output",
		"Enter D/A Channel (0 or 1)",
		"Enter D/A Value (decimal)",
		2,
		doWriteDtoA },
	{ 'O',	
		"Open an adapter", 
		"The adapter number you wish to open (the first adapter in the computer is adapter 0, the second is adapter 1, etc)", 
		" ", 
		1, 
		doOpenPort },
	{ 'R',	
		"Read a byte from a port", 
		"Port from which you would like to read a value", 
		" ", 
		1, 
		doReadPort },
	{ 'S',	
		"Set A/D Input Range (0-5 is the default)",
		"Enter Range (0: 0-5, 1: -5-5, 2: 0-10, 3: -10-10)",
		"",
		1,
		doSetAtoDRange },
#ifndef LINUX
	{ 'V',	
		"Wait for an input to change", 
		" ", 
		" ", 
		0,	
		doWaitForPorts },
#endif
	{ 'W',	
		"Write a byte to a port", 
		"Port to write to", 
		"Data to write to that port (decimal)", 
		2,  
		doWritePort },
	{ 'X',	
		"Start Interrupt Sampling", 
		" ", 
		" ", 
		0,	
		doStartSampling },
	{ 'Y',	
		"Stop Interrupt Sampling", 
		" ", 
		" ", 
		0,	
		doStopSampling },
	{ 'Q',	
		"Quit", 
		" ", 
		" ", 
		0,	
		doQuit },
};

#endif //__TESTIT_H__
